﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Drawing.Imaging;

namespace Kresleni
{
    public partial class Window : Form
    {
        public Window()
        {
            InitializeComponent();
            comboBox.SelectedIndex = 0;
            pictureBox_SizeChanged (null, null);
            p = new Pen (panel1.BackColor);
            b = new SolidBrush (panel2.BackColor);
            c1 = panel2.BackColor;
            c2 = panel2.BackColor;
            align (panel2);
            align (panel3);
        }

        private void pictureBox_SizeChanged(object sender, EventArgs e)
        {
            int w = pictureBox.Width;
            int h = pictureBox.Height;
            if (w > 0 && h > 0)
            {
                Image last = pictureBox.Image;
                if (last != null)
                {
                    w = Math.Max(w, last.Width);
                    h = Math.Max(h, last.Height);
                }
                pictureBox.Image = new Bitmap (w, h);
                Graphics g = Graphics.FromImage (pictureBox.Image);
                Brush b = new SolidBrush (Color.White);
                g.FillRectangle (b, 0, 0, w, h);
                if (last != null)
                    g.DrawImage (last, 0, 0);
            }
        }

        private int X0, Y0;
        private bool pressed = false;
        private Bitmap save;
        private Pen p;
        private Color c1;
        private Color c2;
        private Brush b;
        private int cnt = 1; // pocet barevnych panelu

        // na konec konstruktoru pripsat
        // align (panel2);
        // align (panel3);

        private void align (Panel panel)
        {
            panel.Left = panel1.Width * cnt + panel1.Left * (cnt + 1);
            cnt ++;
            panel.Top = panel1.Top;
            panel.Size = panel1.Size;
        }
            
        private void newColorItem_Click (object sender, EventArgs e)
        {
            Panel panel = new Panel ();
            panel.BackColor = Color.Yellow;
            panel.Parent = toolPanel;
            // toolPanel.Controls.Add (panel);
            align (panel);
            panel.MouseDown += panel1_MouseDown ;
        }

        private void panel1_MouseDown(object sender, MouseEventArgs e)
        {
            Panel panel = (Panel) sender;
            if (Control.ModifierKeys == Keys.Control)
            {
                colorDialog1.Color = panel.BackColor;
                if (colorDialog1.ShowDialog () == DialogResult.OK)
                {
                    panel.BackColor = colorDialog1.Color;
                }
            }
            else if (e.Button == MouseButtons.Left)
            {
                p = new Pen (panel.BackColor);
            }
            else if (e.Button == MouseButtons.Middle)
            {
                c2 = panel.BackColor;
                // using System.Drawing.Drawing2D;
                int x = (int) numericUpDown1.Value;
                int y = (int) numericUpDown2.Value;
                b = new LinearGradientBrush 
                        (new Point (0, 0),
                         new Point (x, y),
                         c1, c2);
            }
            else if (e.Button == MouseButtons.Right)
            {
                c1 = panel.BackColor;
                b = new SolidBrush (c1);
            }
        }

        private void pictureBox_MouseDown(object sender, MouseEventArgs e)
        {
            X0 = e.X;
            Y0 = e.Y;
            pressed = true;
            save = new Bitmap (pictureBox.Image);
        }

        private void pictureBox_MouseMove(object sender, MouseEventArgs e)
        {
            if (pressed)
            {
                Graphics g = Graphics.FromImage (pictureBox.Image);
                g.DrawImage (save, 0, 0);
                int inx = comboBox.SelectedIndex;
                if (e.Button == MouseButtons.Middle) inx = 1;
                else if (e.Button == MouseButtons.Right) inx = 2;
                switch (inx)
                {
                    case 0:
                        g.DrawLine (p, X0, Y0, e.X, e.Y);
                        break;

                    case 1:
                        {
                            int X1 = X0;
                            int Y1 = Y0;
                            int X2 = e.X;
                            int Y2 = e.Y;
                            if (X1 > X2) { int t = X1; X1 = X2; X2 = t; }
                            if (Y1 > Y2) { int t = Y1; Y1 = Y2; Y2 = t; }
                            g.FillRectangle (b, X1, Y1, X2 - X1, Y2 - Y1);
                            g.DrawRectangle (p, X1, Y1, X2 - X1, Y2 - Y1);
                        }
                        break;

                    case 2:
                        g.FillEllipse (b, X0, Y0, e.X - X0, e.Y - Y0);
                        g.DrawEllipse (p, X0, Y0, e.X - X0, e.Y - Y0);
                        break;
                }
                pictureBox.Invalidate ();
            }
        }

        private void pictureBox_MouseUp(object sender, MouseEventArgs e)
        {
            pressed = false;
            // Brush b = new SolidBrush(Color.Blue);
            // g.FillRectangle(b, e.X - 1, e.Y - 1, 3, 3);
        }

        private void openItem_Click (object sender, EventArgs e)
        {
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                pictureBox.Image = new Bitmap (openFileDialog1.FileName);
                pictureBox_SizeChanged (null, null); // zvetseni
                saveFileDialog1.FileName = openFileDialog1.FileName;
            }

        }

        private void saveItem_Click (object sender, EventArgs e)
        {
            if (saveFileDialog1.ShowDialog () == DialogResult.OK)
            {
                int inx = saveFileDialog1.FilterIndex;
                // using System.Drawing.Imaging;
                ImageFormat fmt = ImageFormat.Bmp;
                if (inx == 2)
                    fmt = ImageFormat.Jpeg;
                pictureBox.Image.Save (saveFileDialog1.FileName, fmt);
                openFileDialog1.FileName = saveFileDialog1.FileName;
            }
        }

        private void newPanelItem_Click(object sender, EventArgs e)
        {
            Panel box = new Panel ();
            box.BackColor = Color.Yellow;
            pictureBox.Controls.Add (box);
            box.MouseDown += box_MouseDown;
            box.MouseMove += box_MouseMove;
            box.MouseUp += box_MouseUp;

            Button b = new Button();
            b.Left = 10;
            b.Top = 10;
            b.Text = "Open";
            b.BackColor = SystemColors.ButtonFace;
            b.Click += button_click;
            box.Controls.Add(b);
        }

        private void button_click (object sender, EventArgs e)
        {
            Button b = (Button) sender;
            // b.Text = "Click";
            Panel panel = (Panel) b.Parent;
            // panel.BackColor = Color.Green;
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                panel.BackgroundImage = new Bitmap(openFileDialog1.FileName);
                panel.BackgroundImageLayout = ImageLayout.Zoom;
            }
        }


        private bool down = false;
        private int XS, YS;
        
        private void box_MouseDown(object sender, MouseEventArgs e)
        {
            down = true; XS = e.X; YS = e.Y;
            Panel box = (Panel)sender;
            if (e.Button == MouseButtons.Middle)
            {
                colorDialog1.Color = box.BackColor;
                if (colorDialog1.ShowDialog() == DialogResult.OK)
                    box.BackColor = colorDialog1.Color;
            }
        }
        
        private void box_MouseMove (object sender, MouseEventArgs e)
        {
            if (down)
            {
                Panel box = (Panel) sender;
                if (e.Button == MouseButtons.Left)
                {
                    box.Left += e.X - XS;
                    box.Top += e.Y - YS;
                }
                else if (e.Button == MouseButtons.Right)
                {
                    box.Width += e.X - XS;
                    box.Height += e.Y - YS;
                    XS = e.X;
                    YS = e.Y;
                }
            }
        }
        
        private void box_MouseUp(object sender, MouseEventArgs e)
        {
            down = false;
        }

        private void newAreaToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Area a = new Area ();
            pictureBox.Controls.Add (a);
        }


        private void newWindowItem_Click(object sender, EventArgs e)
        {
            Window w = new Window();
            w.Show ();
        }

        private void quitItem_Click(object sender, EventArgs e)
        {
            Close ();
        }

    }
}
